@@ -1,20 +1,19 @@
-import sys
-import os
 import numpy
-import platform
-from PyQt5.QtWidgets import QApplication
 
 from orangewidget import gui
 from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 
-from orangecontrib.xoppy.util.xoppy_util import locations, XoppyPhysics
-from orangecontrib.xoppy.util.xoppy_xraylib_util import bragg_calc
-
 from oasys.widgets.exchange import DataExchangeObject
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
 
-from xraylib import Crystal_GetCrystalsList
+
+import scipy.constants as codata
+
+import xraylib
+from dabax.dabax_xraylib import DabaxXraylib
+
+from xoppylib.crystals.tools import run_diff_pat, bragg_calc2
 
 class OWxcrystal(XoppyWidget):
     name = "CRYSTAL"
@@ -48,6 +47,14 @@ class OWxcrystal(XoppyWidget):
     CUT = Setting("2 -1 -1 ; 1 1 1 ; 0 0 0")
     FILECOMPLIANCE = Setting("mycompliance.dat")
 
+    # new crystals  #todo: add to menus?
+    material_constants_library_flag = Setting(2) # 0=xraylib, 1=dabax, 2=xraylib completed by dabax
+    dx = None # DABAX object
+
+
+    def __init__(self):
+        super().__init__(show_script_tab=True)
+
     def build_gui(self):
 
         box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
@@ -59,7 +66,7 @@ class OWxcrystal(XoppyWidget):
         box1 = gui.widgetBox(box) 
         gui.comboBox(box1, self, "CRYSTAL_MATERIAL",
                      label=self.unitLabels()[idx], addSpace=False,
-                    items=Crystal_GetCrystalsList(),
+                    items=self.get_crystal_list(),
                     valueType=int, orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1) 
         
@@ -301,12 +308,279 @@ class OWxcrystal(XoppyWidget):
             elif self.ANISOTROPY == 3:
                 congruence.checkFile(self.FILECOMPLIANCE)
 
+    def get_crystal_list(self):
+        crystal_list_xrl = list(xraylib.Crystal_GetCrystalsList())
+
+        if self.material_constants_library_flag == 0:
+            return crystal_list_xrl
+
+        if self.dx is None:
+            self.dx = DabaxXraylib()
+        crystal_list_dabax = self.dx.Crystal_GetCrystalsList()
+
+        if self.material_constants_library_flag == 1:
+            return crystal_list_dabax
+
+        crystal_list_combined = crystal_list_xrl
+        for crystal in crystal_list_dabax:
+            if crystal not in crystal_list_combined:
+                crystal_list_combined.append(crystal)
+
+        if self.material_constants_library_flag == 2:
+            return crystal_list_combined
 
     def do_xoppy_calculation(self):
-        return self.xoppy_calc_xcrystal()
+        # return self.xoppy_calc_xcrystal()
+
+        descriptor = self.get_crystal_list()[self.CRYSTAL_MATERIAL]
+
+        if self.material_constants_library_flag == 0:
+            material_constants_library = xraylib
+        elif self.material_constants_library_flag == 1:
+            material_constants_library = self.dx
+        elif self.material_constants_library_flag == 2:
+            if descriptor in xraylib.Crystal_GetCrystalsList():
+                material_constants_library = xraylib
+            elif descriptor in self.dx.Crystal_GetCrystalsList():
+                material_constants_library = self.dx
+            else:
+                raise Exception("Descriptor not found in material constants database")
+
+        #
+        # run bragg_calc (preprocessor) and create file xcrystal.bra
+        #
+
+        # for file in ["xcrystal.bra"]:
+        #     try:
+        #         os.remove(os.path.join(locations.home_bin_run(), file))
+        #     except:
+        #         pass
+
+        if self.SCAN == 3:  # energy scan
+            emin = self.SCANFROM - 1
+            emax = self.SCANTO + 1
+        else:
+            emin = self.ENERGY - 100.0
+            emax = self.ENERGY + 100.0
+
+        estep = (emax - emin) / self.SCANPOINTS
+        preprocessor_file = "xcrystal.bra"
+
+        print("Using crystal descriptor: ", descriptor)
+
+        bragg_dictionary = bragg_calc2(
+            descriptor=descriptor,
+            hh=self.MILLER_INDEX_H,
+            kk=self.MILLER_INDEX_K,
+            ll=self.MILLER_INDEX_L,
+            temper=float(self.TEMPER),
+            emin=emin,
+            emax=emax,
+            estep=estep,
+            ANISO_SEL=0,
+            fileout=preprocessor_file,
+            do_not_prototype=0,  # 0=use site groups (recommended), 1=use all individual sites
+            verbose=False,
+            material_constants_library=material_constants_library,
+        )
+
+        #
+        # run external (fortran) diff_pat
+        #
+        run_diff_pat(
+            bragg_dictionary,
+            preprocessor_file=preprocessor_file,
+            descriptor=descriptor,
+            MOSAIC=self.MOSAIC,
+            GEOMETRY=self.GEOMETRY,
+            SCAN=self.SCAN,
+            UNIT=self.UNIT,
+            SCANFROM=self.SCANFROM,
+            SCANTO=self.SCANTO,
+            SCANPOINTS=self.SCANPOINTS,
+            ENERGY=self.ENERGY,
+            ASYMMETRY_ANGLE=self.ASYMMETRY_ANGLE,
+            THICKNESS=self.THICKNESS,
+            MOSAIC_FWHM=self.MOSAIC_FWHM,
+            RSAG=self.RSAG,
+            RMER=self.RMER,
+            ANISOTROPY=self.ANISOTROPY,
+            POISSON=self.POISSON,
+            CUT=self.CUT,
+            FILECOMPLIANCE=self.FILECOMPLIANCE,
+        )
+
+        # show calculated parameters in standard output
+        txt_info = open("diff_pat.par").read()
+        for line in txt_info:
+            print(line, end="")
+
+        #
+        # write python script
+        #
+        if isinstance(material_constants_library, DabaxXraylib):
+            material_constants_library_txt = "DabaxXraylib()"
+        else:
+            material_constants_library_txt = "xraylib"
+
+        dict_parameters = {
+            'CRYSTAL_DESCRIPTOR': descriptor,
+            'MILLER_INDEX_H': self.MILLER_INDEX_H,
+            'MILLER_INDEX_K': self.MILLER_INDEX_K,
+            'MILLER_INDEX_L': self.MILLER_INDEX_L,
+            'TEMPER': self.TEMPER,
+            'MOSAIC': self.MOSAIC,
+            'GEOMETRY': self.GEOMETRY,
+            'SCAN': self.SCAN,
+            'UNIT': self.UNIT,
+            'SCANFROM': self.SCANFROM,
+            'SCANTO': self.SCANTO,
+            'SCANPOINTS': self.SCANPOINTS,
+            'ENERGY': self.ENERGY,
+            'ASYMMETRY_ANGLE': self.ASYMMETRY_ANGLE,
+            'THICKNESS': self.THICKNESS,
+            'MOSAIC_FWHM': self.MOSAIC_FWHM,
+            'RSAG': self.RSAG,
+            'RMER': self.RMER,
+            'ANISOTROPY': self.ANISOTROPY,
+            'POISSON': self.POISSON,
+            'CUT': self.CUT,
+            'FILECOMPLIANCE': self.FILECOMPLIANCE,
+            'material_constants_library_txt': material_constants_library_txt,
+            'emin': emin,
+            'emax': emax,
+            'estep': estep,
+            'preprocessor_file': preprocessor_file}
+
+        script = self.script_template().format_map(dict_parameters)
+        self.xoppy_script.set_code(script)
+
+        return bragg_dictionary, "diff_pat.dat", script
+
+    def get_units_to_degrees(self):
+        if self.UNIT == 0:  # RADIANS
+            return 57.2957795
+        elif self.UNIT == 1:  # MICRORADIANS
+            return 57.2957795e-6
+        elif self.UNIT == 2:  # DEGREES
+            return 1.0
+        elif self.UNIT == 3:  # ARCSEC
+            return 0.000277777805
+
+    def script_template(self):
+        return """
+#
+# script to calculate crystal diffraction profiles (created by XOPPY:crystal)
+#
+
+import numpy
+from xoppylib.crystals.tools import bragg_calc2, run_diff_pat
+import xraylib
+from dabax.dabax_xraylib import DabaxXraylib
+
+#
+# run bragg_calc (preprocessor) and create file xcrystal.bra
+#
+bragg_dictionary = bragg_calc2(
+    descriptor = "{CRYSTAL_DESCRIPTOR}",
+    hh         = {MILLER_INDEX_H}, 
+    kk         = {MILLER_INDEX_K}, 
+    ll         = {MILLER_INDEX_L}, 
+    temper     = {TEMPER}, 
+    emin       = {emin},
+    emax       = {emax},
+    estep      = {estep},
+    ANISO_SEL  = 0,
+    fileout    = "{preprocessor_file}",
+    do_not_prototype = 0,  # 0=use site groups (recommended), 1=use all individual sites
+    verbose = False,
+    material_constants_library = {material_constants_library_txt},
+    )
+
+#
+# run external (fortran) diff_pat (note that some parameters may not be used)
+#
+run_diff_pat( 
+    bragg_dictionary,
+    preprocessor_file  = "{preprocessor_file}",
+    descriptor         = "{CRYSTAL_DESCRIPTOR}",
+    MOSAIC             = {MOSAIC}, 
+    GEOMETRY           = {GEOMETRY}, 
+    SCAN               = {SCAN}, 
+    UNIT               = {UNIT}, 
+    SCANFROM           = {SCANFROM}, 
+    SCANTO             = {SCANTO}, 
+    SCANPOINTS         = {SCANPOINTS}, 
+    ENERGY             = {ENERGY}, 
+    ASYMMETRY_ANGLE    = {ASYMMETRY_ANGLE}, 
+    THICKNESS          = {THICKNESS}, 
+    MOSAIC_FWHM        = {MOSAIC_FWHM}, 
+    RSAG               = {RSAG}, 
+    RMER               = {RMER}, 
+    ANISOTROPY         = {ANISOTROPY}, 
+    POISSON            = {POISSON}, 
+    CUT                = "{CUT}",
+    FILECOMPLIANCE     = "{FILECOMPLIANCE}", 
+    )
+
+#                       
+# example plot
+#
+from srxraylib.plot.gol import plot
+data = numpy.loadtxt("diff_pat.dat", skiprows=5)
+plot(data[:,0], data[:,-1], data[:,0], data[:,-2], ytitle='Crystal reflectivity', legend=['s-polarization','p-polarization'])
+
+#
+# end script
+#
+"""
 
     def extract_data_from_xoppy_output(self, calculation_output):
-        return calculation_output
+        #
+        # prepare outputs
+        #
+        bragg_dictionary, diff_pat_file, script = calculation_output
+
+        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
+
+        try:
+            calculated_data.add_content("xoppy_data", numpy.loadtxt(diff_pat_file, skiprows=5))
+            calculated_data.add_content("plot_x_col", 0)
+            calculated_data.add_content("plot_y_col", -1)
+            calculated_data.add_content("scan_type", self.SCAN)
+
+            if self.SCAN in (1, 2):
+                wavelength = codata.h * codata.c / codata.e / self.ENERGY * 1e2  # cm
+                dspacing = float(bragg_dictionary["dspacing"])
+
+                calculated_data.add_content("bragg_angle", numpy.degrees(numpy.arcsin(wavelength / (2 * dspacing))))
+                calculated_data.add_content("asymmetry_angle", self.ASYMMETRY_ANGLE)
+
+            calculated_data.add_content("units_to_degrees", self.get_units_to_degrees())
+        except Exception as e:
+            raise Exception("Error loading diff_pat.dat :" + str(e))
+
+        try:
+            calculated_data.add_content("labels",
+                                        ["Th-ThB{in} [" + self.unit_combo.itemText(self.UNIT) + "]",
+                                         "Th-ThB{out} [" + self.unit_combo.itemText(self.UNIT) + "]",
+                                         "phase_p[rad]",
+                                         "phase_s[rad]", "Circ Polariz",
+                                         "p-polarized reflectivity",
+                                         "s-polarized reflectivity"])
+
+        except:
+            pass
+
+        try:
+            with open("diff_pat.par") as f:
+                info = f.readlines()
+            calculated_data.add_content("info", info)
+        except:
+            pass
+
+        return calculated_data
+
 
     def get_data_exchange_widget_name(self):
         return "XCRYSTAL"
@@ -370,216 +644,265 @@ class OWxcrystal(XoppyWidget):
                                                           text="Peak=%5.2f"%(y[index_ymax]), color="pink",
                                                           selectable=False, draggable=False, symbol=None, constraint=None)
 
-    def xoppy_calc_xcrystal(self):
-        CRYSTAL_MATERIAL = self.CRYSTAL_MATERIAL
-        MILLER_INDEX_H = self.MILLER_INDEX_H
-        MILLER_INDEX_K = self.MILLER_INDEX_K
-        MILLER_INDEX_L = self.MILLER_INDEX_L
-        TEMPER = self.TEMPER
-        MOSAIC = self.MOSAIC
-        GEOMETRY = self.GEOMETRY
-        SCAN = self.SCAN
-        UNIT = self.UNIT
-        SCANFROM = self.SCANFROM
-        SCANTO = self.SCANTO
-        SCANPOINTS = self.SCANPOINTS
-        ENERGY = self.ENERGY
-        ASYMMETRY_ANGLE = self.ASYMMETRY_ANGLE
-        THICKNESS = self.THICKNESS
-        MOSAIC_FWHM = self.MOSAIC_FWHM
-        RSAG = self.RSAG
-        RMER = self.RMER
-        ANISOTROPY = self.ANISOTROPY
-        POISSON = self.POISSON
-        CUT = self.CUT
-        FILECOMPLIANCE = self.FILECOMPLIANCE
-
-
-        for file in ["diff_pat.dat","diff_pat.gle","diff_pat.par","diff_pat.xop","xcrystal.bra"]:
-            try:
-                os.remove(os.path.join(locations.home_bin_run(),file))
-            except:
-                pass
-
-
-        if (GEOMETRY == 1) or (GEOMETRY == 3):
-            if ASYMMETRY_ANGLE == 0.0:
-                print("xoppy_calc_xcrystal: WARNING: In xcrystal the asymmetry angle is the angle between Bragg planes and crystal surface,"+
-                      "in BOTH Bragg and Laue geometries.")
-
-
-        descriptor = Crystal_GetCrystalsList()[CRYSTAL_MATERIAL]
-
-        if SCAN == 3: # energy scan
-            emin = SCANFROM - 1
-            emax = SCANTO + 1
-        else:
-            emin = ENERGY - 100.0
-            emax = ENERGY + 100.0
-
-        print("Using crystal descriptor: ",descriptor)
-
-        bragg_dictionary = bragg_calc(descriptor=descriptor,
-                                                hh=MILLER_INDEX_H,kk=MILLER_INDEX_K,ll=MILLER_INDEX_L,
-                                                temper=float(TEMPER),
-                                                emin=emin,emax=emax,estep=5.0,fileout="xcrystal.bra")
-
-        with open("xoppy.inp", "wt") as f:
-            f.write("xcrystal.bra\n")
-            f.write("%d\n"%MOSAIC)
-            f.write("%d\n"%GEOMETRY)
-
-            if MOSAIC == 1:
-                f.write("%g\n"%MOSAIC_FWHM)
-                f.write("%g\n"%THICKNESS)
-            else:
-                f.write("%g\n"%THICKNESS)
-                f.write("%g\n"%ASYMMETRY_ANGLE)
-
-            scan_flag = 1 + SCAN
-
-            f.write("%d\n"%scan_flag)
-
-            f.write("%19.9f\n"%ENERGY)
-
-            if scan_flag <= 3:
-                f.write("%d\n"%UNIT)
-
-            f.write("%g\n"%SCANFROM)
-            f.write("%g\n"%SCANTO)
-            f.write("%d\n"%SCANPOINTS)
-
-            if MOSAIC > 1: # bent
-                f.write("%g\n"%RSAG)
-                f.write("%g\n"%RMER)
-                f.write("0\n")
-
-                if ( (descriptor == "Si") or (descriptor == "Si2") or (descriptor == "Si_NIST") or (descriptor == "Ge") or descriptor == "Diamond"):
-                    pass
-                else:  # not Si,Ge,Diamond
-                    if ((ANISOTROPY == 1) or (ANISOTROPY == 2)):
-                        raise Exception("Anisotropy data not available for this crystal. Either use isotropic or use external compliance file. Please change and run again'")
-
-                f.write("%d\n"%ANISOTROPY)
-
-                if ANISOTROPY == 0:
-                    f.write("%g\n"%POISSON)
-                elif ANISOTROPY == 1:
-                    # elas%crystalindex =  irint('CrystalIndex: 0,1,2=Si,3=Ge,4=Diamond: ')
-                    if descriptor == "Si":
-                        f.write("0\n")
-                    elif descriptor == "Si2":
-                        f.write("1\n")
-                    elif descriptor == "Si_NIST":
-                        f.write("2\n")
-                    elif descriptor == "Ge":
-                        f.write("3\n")
-                    elif descriptor == "Diamond":
-                        f.write("4\n")
-                    else:
-                        raise Exception("Cannot calculate anisotropy data for %s this crystal. Use Si, Ge, Diamond." % descriptor)
-
-                    f.write("%g\n"%ASYMMETRY_ANGLE)
-                    f.write("%d\n"%MILLER_INDEX_H)
-                    f.write("%d\n"%MILLER_INDEX_K)
-                    f.write("%d\n"%MILLER_INDEX_L)
-                elif ANISOTROPY == 2:
-                    # elas%crystalindex =  irint('CrystalIndex: 0,1,2=Si,3=Ge,4=Diamond: ')
-                    if descriptor == "Si":
-                        f.write("0\n")
-                    elif descriptor == "Si2":
-                        f.write("1\n")
-                    elif descriptor == "Si_NIST":
-                        f.write("2\n")
-                    elif descriptor == "Ge":
-                        f.write("3\n")
-                    elif descriptor == "Diamond":
-                        f.write("4\n")
-                    else:
-                        raise Exception("Cannot calculate anisotropy data for %s this crystal. Use Si, Ge, Diamond." % descriptor)
-
-                    f.write("%g\n"%ASYMMETRY_ANGLE)
-                    # TODO: check syntax for CUT: Cut syntax is: valong_X valong_Y valong_Z ; vnorm_X vnorm_Y vnorm_Z ; vperp_x vperp_Y vperp_Z
-                    f.write("%s\n"%CUT.split(";")[0])
-                    f.write("%s\n"%CUT.split(";")[1])
-                    f.write("%s\n"%CUT.split(";")[2])
-                elif ANISOTROPY == 3:
-                    f.write("%s\n"%FILECOMPLIANCE)
-
-
-
-        if platform.system() == "Windows":
-            command = "\"" + os.path.join(locations.home_bin(), 'diff_pat.exe\" < xoppy.inp')
-        else:
-            command = "'" + os.path.join(locations.home_bin(), 'diff_pat') + "' < xoppy.inp"
-        print("Running command '%s' in directory: %s "%(command, locations.home_bin_run()))
-        print("\n--------------------------------------------------------\n")
-        os.system(command)
-        print("\n--------------------------------------------------------\n")
-        
-        #show calculated parameters in standard output
-        txt_info = open("diff_pat.par").read()
-        for line in txt_info:
-            print(line,end="")
-
-
-        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
-
-        try:
-            calculated_data.add_content("xoppy_data", numpy.loadtxt("diff_pat.dat", skiprows=5))
-            calculated_data.add_content("plot_x_col",0)
-            calculated_data.add_content("plot_y_col",-1)
-            calculated_data.add_content("scan_type", SCAN)
-
-            if SCAN in (1, 2):
-                wavelength = XoppyPhysics.getWavelengthFromEnergy(self.ENERGY) * 1e-8 #cm
-                dspacing = float(bragg_dictionary["dspacing"])
-
-                calculated_data.add_content("bragg_angle", numpy.degrees(numpy.arcsin(wavelength/(2*dspacing))))
-                calculated_data.add_content("asymmetry_angle", self.ASYMMETRY_ANGLE)
-
-            calculated_data.add_content("units_to_degrees", self.get_units_to_degrees())
-        except Exception as e:
-            raise Exception("Error loading diff_pat.dat :" + str(e))
-
-        try:
-            calculated_data.add_content("labels",
-                                        ["Th-ThB{in} [" + self.unit_combo.itemText(self.UNIT) + "]",
-                                         "Th-ThB{out} [" + self.unit_combo.itemText(self.UNIT) + "]",
-                                         "phase_p[rad]",
-                                         "phase_s[rad]","Circ Polariz",
-                                         "p-polarized reflectivity",
-                                         "s-polarized reflectivity"])
-
-        except:
-            pass
-
-        try:
-            with open("diff_pat.par") as f:
-                info = f.readlines()
-            calculated_data.add_content("info",info)
-        except:
-            pass
-
-        return calculated_data
-
-
-    def get_units_to_degrees(self):
-        if self.UNIT == 0: # RADIANS
-            return 57.2957795
-        elif self.UNIT == 1: #MICRORADIANS
-            return 57.2957795e-6
-        elif self.UNIT == 2: # DEGREES
-            return 1.0
-        elif self.UNIT == 3: # ARCSEC
-            return 0.000277777805
-
-
-
-
-
+#     def xoppy_calc_xcrystal(self):
+#
+#         descriptor = self.get_crystal_list()[self.CRYSTAL_MATERIAL]
+#         print(">>>>>> descriptor 0: ", descriptor)
+#
+#         if self.material_constants_library_flag == 0:
+#             material_constants_library = xraylib
+#         elif self.material_constants_library_flag == 1:
+#             material_constants_library = self.dx
+#         elif self.material_constants_library_flag == 2:
+#             if descriptor in xraylib.Crystal_GetCrystalsList():
+#                 material_constants_library = xraylib
+#             elif descriptor in self.dx.Crystal_GetCrystalsList():
+#                 material_constants_library = self.dx
+#             else:
+#                 raise Exception("Descriptor not found in material constants database")
+#
+#         #
+#         # run bragg_calc (preprocessor) and create file xcrystal.bra
+#         #
+#
+#         # for file in ["xcrystal.bra"]:
+#         #     try:
+#         #         os.remove(os.path.join(locations.home_bin_run(), file))
+#         #     except:
+#         #         pass
+#
+#         if self.SCAN == 3:  # energy scan
+#             emin = self.SCANFROM - 1
+#             emax = self.SCANTO + 1
+#         else:
+#             emin = self.ENERGY - 100.0
+#             emax = self.ENERGY + 100.0
+#
+#         estep = (emax - emin) / self.SCANPOINTS
+#         preprocessor_file = "xcrystal.bra"
+#
+#         print("Using crystal descriptor: ", descriptor)
+#
+#         bragg_dictionary = bragg_calc2(
+#             descriptor=descriptor,
+#             hh = self.MILLER_INDEX_H,
+#             kk = self.MILLER_INDEX_K,
+#             ll = self.MILLER_INDEX_L,
+#             temper = float(self.TEMPER),
+#             emin=emin,
+#             emax=emax,
+#             estep=estep,
+#             ANISO_SEL=0,
+#             fileout=preprocessor_file,
+#             do_not_prototype=0,  # 0=use site groups (recommended), 1=use all individual sites
+#             verbose=False,
+#             material_constants_library=material_constants_library,
+#         )
+#
+#         print(">>>>>> descriptor 1: ", descriptor)
+#         #
+#         # run external (fortran) diff_pat
+#         #
+#         run_diff_pat(
+#             bragg_dictionary,
+#             preprocessor_file=preprocessor_file,
+#             descriptor=descriptor,
+#             MOSAIC = self.MOSAIC,
+#             GEOMETRY = self.GEOMETRY,
+#             SCAN = self.SCAN,
+#             UNIT = self.UNIT,
+#             SCANFROM = self.SCANFROM,
+#             SCANTO = self.SCANTO,
+#             SCANPOINTS = self.SCANPOINTS,
+#             ENERGY = self.ENERGY,
+#             ASYMMETRY_ANGLE = self.ASYMMETRY_ANGLE,
+#             THICKNESS = self.THICKNESS,
+#             MOSAIC_FWHM = self.MOSAIC_FWHM,
+#             RSAG = self.RSAG,
+#             RMER = self.RMER,
+#             ANISOTROPY = self.ANISOTROPY,
+#             POISSON = self.POISSON,
+#             CUT = self.CUT,
+#             FILECOMPLIANCE = self.FILECOMPLIANCE,
+#             )
+#
+#         print(">>>>>> descriptor 2: ", descriptor)
+#         #show calculated parameters in standard output
+#         txt_info = open("diff_pat.par").read()
+#         for line in txt_info:
+#             print(line,end="")
+#
+#         #
+#         # write python script
+#         #
+#         if isinstance(material_constants_library, DabaxXraylib):
+#             material_constants_library_txt = "DabaxXraylib()"
+#         else:
+#             material_constants_library_txt = "xraylib"
+#
+#
+#         dict_parameters = {
+#             'CRYSTAL_DESCRIPTOR': descriptor,
+#             'MILLER_INDEX_H': self.MILLER_INDEX_H,
+#             'MILLER_INDEX_K': self.MILLER_INDEX_K,
+#             'MILLER_INDEX_L': self.MILLER_INDEX_L,
+#             'TEMPER': self.TEMPER,
+#             'MOSAIC': self.MOSAIC,
+#             'GEOMETRY': self.GEOMETRY,
+#             'SCAN': self.SCAN,
+#             'UNIT': self.UNIT,
+#             'SCANFROM': self.SCANFROM,
+#             'SCANTO': self.SCANTO,
+#             'SCANPOINTS': self.SCANPOINTS,
+#             'ENERGY': self.ENERGY,
+#             'ASYMMETRY_ANGLE': self.ASYMMETRY_ANGLE,
+#             'THICKNESS': self.THICKNESS,
+#             'MOSAIC_FWHM': self.MOSAIC_FWHM,
+#             'RSAG': self.RSAG,
+#             'RMER': self.RMER,
+#             'ANISOTROPY': self.ANISOTROPY,
+#             'POISSON': self.POISSON,
+#             'CUT': self.CUT,
+#             'FILECOMPLIANCE': self.FILECOMPLIANCE,
+#             'material_constants_library_txt': material_constants_library_txt,
+#             'emin': emin,
+#             'emax': emax,
+#             'estep': estep,
+#             'preprocessor_file': preprocessor_file}
+#
+#         self.xoppy_script.set_code(self.script_template().format_map(dict_parameters))
+#
+#
+#         #
+#         # prepare outputs
+#         #
+#
+#         calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
+#
+#         try:
+#             calculated_data.add_content("xoppy_data", numpy.loadtxt("diff_pat.dat", skiprows=5))
+#             calculated_data.add_content("plot_x_col",0)
+#             calculated_data.add_content("plot_y_col",-1)
+#             calculated_data.add_content("scan_type", self.SCAN)
+#
+#             if self.SCAN in (1, 2):
+#                 wavelength = codata.h * codata.c / codata.e / self.ENERGY * 1e2 # cm
+#                 dspacing = float(bragg_dictionary["dspacing"])
+#
+#                 calculated_data.add_content("bragg_angle", numpy.degrees(numpy.arcsin(wavelength/(2*dspacing))))
+#                 calculated_data.add_content("asymmetry_angle", self.ASYMMETRY_ANGLE)
+#
+#             calculated_data.add_content("units_to_degrees", self.get_units_to_degrees())
+#         except Exception as e:
+#             raise Exception("Error loading diff_pat.dat :" + str(e))
+#
+#         try:
+#             calculated_data.add_content("labels",
+#                                         ["Th-ThB{in} [" + self.unit_combo.itemText(self.UNIT) + "]",
+#                                          "Th-ThB{out} [" + self.unit_combo.itemText(self.UNIT) + "]",
+#                                          "phase_p[rad]",
+#                                          "phase_s[rad]","Circ Polariz",
+#                                          "p-polarized reflectivity",
+#                                          "s-polarized reflectivity"])
+#
+#         except:
+#             pass
+#
+#         try:
+#             with open("diff_pat.par") as f:
+#                 info = f.readlines()
+#             calculated_data.add_content("info",info)
+#         except:
+#             pass
+#
+#         return calculated_data
+#
+#
+#     def get_units_to_degrees(self):
+#         if self.UNIT == 0: # RADIANS
+#             return 57.2957795
+#         elif self.UNIT == 1: #MICRORADIANS
+#             return 57.2957795e-6
+#         elif self.UNIT == 2: # DEGREES
+#             return 1.0
+#         elif self.UNIT == 3: # ARCSEC
+#             return 0.000277777805
+#
+#     def script_template(self):
+#         return """
+# #
+# # script to calculate crystal diffraction profiles (created by XOPPY:crystal)
+# #
+#
+# import numpy
+# from xoppylib.crystals.tools import bragg_calc2, run_diff_pat
+# import xraylib
+# from dabax.dabax_xraylib import DabaxXraylib
+#
+# #
+# # run bragg_calc (preprocessor) and create file xcrystal.bra
+# #
+# bragg_dictionary = bragg_calc2(
+#     descriptor = "{CRYSTAL_DESCRIPTOR}",
+#     hh         = {MILLER_INDEX_H},
+#     kk         = {MILLER_INDEX_K},
+#     ll         = {MILLER_INDEX_L},
+#     temper     = {TEMPER},
+#     emin       = {emin},
+#     emax       = {emax},
+#     estep      = {estep},
+#     ANISO_SEL  = 0,
+#     fileout    = "{preprocessor_file}",
+#     do_not_prototype = 0,  # 0=use site groups (recommended), 1=use all individual sites
+#     verbose = False,
+#     material_constants_library = {material_constants_library_txt},
+#     )
+#
+# #
+# # run external (fortran) diff_pat (note that some parameters may not be used)
+# #
+# run_diff_pat(
+#     bragg_dictionary,
+#     preprocessor_file  = "{preprocessor_file}",
+#     descriptor         = "{CRYSTAL_DESCRIPTOR}",
+#     MOSAIC             = {MOSAIC},
+#     GEOMETRY           = {GEOMETRY},
+#     SCAN               = {SCAN},
+#     UNIT               = {UNIT},
+#     SCANFROM           = {SCANFROM},
+#     SCANTO             = {SCANTO},
+#     SCANPOINTS         = {SCANPOINTS},
+#     ENERGY             = {ENERGY},
+#     ASYMMETRY_ANGLE    = {ASYMMETRY_ANGLE},
+#     THICKNESS          = {THICKNESS},
+#     MOSAIC_FWHM        = {MOSAIC_FWHM},
+#     RSAG               = {RSAG},
+#     RMER               = {RMER},
+#     ANISOTROPY         = {ANISOTROPY},
+#     POISSON            = {POISSON},
+#     CUT                = "{CUT}",
+#     FILECOMPLIANCE     = "{FILECOMPLIANCE}",
+#     )
+#
+# #
+# # example plot
+# #
+# from srxraylib.plot.gol import plot
+# data = numpy.loadtxt("diff_pat.dat", skiprows=5)
+# plot(data[:,0], data[:,-1], data[:,0], data[:,-2], ytitle='Crystal reflectivity', legend=['s-polarization','p-polarization'])
+#
+# #
+# # end script
+# #
+# """
 
+#
+#
+#
 if __name__ == "__main__":
+    import sys
+    from PyQt5.QtWidgets import QApplication
+
     app = QApplication(sys.argv)
     w = OWxcrystal()
     w.show()
