@@ -1,4 +1,3 @@
-import sys
 import numpy
 from PyQt5.QtWidgets import QApplication, QMessageBox, QSizePolicy
 
@@ -6,14 +5,17 @@ from orangewidget import gui
 from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 from oasys.widgets.exchange import DataExchangeObject
-from orangecontrib.xoppy.util.xoppy_xraylib_util import xpower_calc
+
+from xoppylib.power.xoppy_calc_power import xoppy_calc_power
 
 from oasys.widgets.exchange import DataExchangeObject
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
 
-
 import scipy.constants as codata
 
+import xraylib
+from dabax.dabax_xraylib import DabaxXraylib
+
 class OWxpower(XoppyWidget):
     name = "POWER"
     id = "orange.widgets.dataxpower"
@@ -64,6 +66,14 @@ class OWxpower(XoppyWidget):
     PLOT_SETS = Setting(2)
     FILE_DUMP = 0
 
+    MATERIAL_CONSTANT_LIBRARY_FLAG = Setting(0) # not yet interfaced, to be done
+
+    input_spectrum = None
+    input_script = None
+
+    def __init__(self):
+        super().__init__(show_script_tab=True)
+
     def build_gui(self):
 
         self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
@@ -79,7 +89,7 @@ class OWxpower(XoppyWidget):
         box1 = gui.widgetBox(box) 
         self.box_source = gui.comboBox(box1, self, "SOURCE",
                      label=self.unitLabels()[idx], addSpace=False,
-                    items=['From Oasys wire', 'Normalized to 1', 'From external file.                '],
+                    items=['From Oasys wire', 'Normalized to 1 W/eV', 'From external file (eV, W/eV)', 'From external file (eV, phot/s/.1%bw)'],
                     valueType=int, orientation="horizontal", labelWidth=150)
         self.show_at(self.unitFlags()[idx], box1)
         
@@ -105,17 +115,7 @@ class OWxpower(XoppyWidget):
         oasysgui.lineEdit(box1, self, "ENER_N",
                      label=self.unitLabels()[idx], addSpace=False,
                     valueType=int, orientation="horizontal", labelWidth=250)
-        self.show_at(self.unitFlags()[idx], box1) 
-        
-        # #widget index 9
-        # idx += 1
-        # box1 = gui.widgetBox(box)
-        #
-        # file_box = oasysgui.widgetBox(box1, "", addSpace=False, orientation="horizontal", height=25)
-        #
-        # self.le_file = oasysgui.lineEdit(file_box, self, "SOURCE_FILE",
-        #              label=self.unitLabels()[idx], addSpace=False, orientation="horizontal")
-        # self.show_at(self.unitFlags()[idx], box1)
+        self.show_at(self.unitFlags()[idx], box1)
 
         #widget index 9 ***********   File Browser ******************
         idx += 1
@@ -402,7 +402,7 @@ class OWxpower(XoppyWidget):
                     valueType=int, orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1)
 
-        self.input_spectrum = None
+        # self.input_spectrum = None
 
     def select_input_file(self):
         self.file_id.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE,
@@ -435,7 +435,7 @@ class OWxpower(XoppyWidget):
                  'self.SOURCE  ==  1',
                  'self.SOURCE  ==  1',
                  'self.SOURCE  ==  1',
-                 'self.SOURCE  ==  2',
+                 'self.SOURCE  >  1',
                  'True',
                  'self.NELEMENTS  >=  0',' self.NELEMENTS  >=  0','self.EL1_FLAG  ==  0  and  self.NELEMENTS  >=  0','self.EL1_FLAG  !=  0  and  self.NELEMENTS  >=  0','self.EL1_FLAG  !=  0  and  self.NELEMENTS  >=  0',' self.NELEMENTS  >=  0',
                  'self.NELEMENTS  >=  1',' self.NELEMENTS  >=  1','self.EL2_FLAG  ==  0  and  self.NELEMENTS  >=  1','self.EL2_FLAG  !=  0  and  self.NELEMENTS  >=  1','self.EL2_FLAG  !=  0  and  self.NELEMENTS  >=  1',' self.NELEMENTS  >=  1',
@@ -453,8 +453,8 @@ class OWxpower(XoppyWidget):
     def acceptExchangeData(self, exchangeData):
 
         self.input_spectrum = None
+        self.input_script = None
         self.SOURCE = 0
-        # self.box_source.setCurrentIndex(self.SOURCE)
 
         try:
             if not exchangeData is None:
@@ -467,12 +467,12 @@ class OWxpower(XoppyWidget):
                     elif exchangeData.get_widget_name() == "BM" :
                         if exchangeData.get_content("is_log_plot") == 1:
                             raise Exception("Logaritmic X scale of Xoppy Energy distribution not supported")
-                        if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") == 0:
+                        if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") in [0,2]:
                             # self.SOURCE_FILE = "xoppy_bm_flux"
                             no_bandwidth = True
                             index_flux = 6
                         else:
-                            raise Exception("Xoppy result is not an Flux vs Energy distribution integrated in Psi")
+                            raise Exception("Xoppy result is not a Flux vs Energy distribution integrated in Psi")
                     elif exchangeData.get_widget_name() =="XWIGGLER" :
                         # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                         no_bandwidth = True
@@ -529,18 +529,16 @@ class OWxpower(XoppyWidget):
 
                         self.input_spectrum = numpy.vstack((spectrum[:,0],spectrum[:,index_flux]))
 
+                    try:
+                        self.input_script = exchangeData.get_content("xoppy_script")
+                    except:
+                        self.input_script = None
+
                     self.process_showers()
                     self.compute()
 
         except Exception as exception:
-            QMessageBox.critical(self, "Error",
-                                       str(exception),
-                QMessageBox.Ok)
-
-            #raise exception
-
-
-
+            QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)
 
     def check_fields(self):
 
@@ -614,10 +612,202 @@ class OWxpower(XoppyWidget):
                 self.EL5_DEN = str(congruence.checkStrictlyPositiveNumber(float(congruence.checkNumber(self.EL5_DEN, "5th oe density")), "5th oe density"))
 
     def do_xoppy_calculation(self):
-        return self.xoppy_calc_xpower()
+
+        if self.SOURCE == 0:
+            if self.input_spectrum is None:
+                raise Exception("No input beam")
+            else:
+                energies = self.input_spectrum[0,:].copy()
+                source = self.input_spectrum[1,:].copy()
+            if self.input_script is None:
+                script_previous = '#\n# >> MISSING SCRIPT TO CREATE (energy, spectral_power) <<\n#\n'
+            else:
+                script_previous = self.input_script
+        elif self.SOURCE == 1:
+            energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
+            source = numpy.ones(energies.size)
+            tmp = numpy.vstack( (energies,source))
+            self.input_spectrum = source
+            script_previous = "import numpy\nenergy = numpy.linspace(%g,%g,%d)\nspectral_power = numpy.ones(%d)\n" % \
+                        (self.ENER_MIN,self.ENER_MAX,self.ENER_N,self.ENER_N)
+        elif self.SOURCE == 2:  # file contains energy_eV and spectral power (W/eV)
+            source_file = self.SOURCE_FILE
+            try:
+                tmp = numpy.loadtxt(source_file)
+                energies = tmp[:,0]
+                source = tmp[:,1]
+                self.input_spectrum = source
+                script_previous = "import numpy\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1]\n" % \
+                                (source_file)
+            except:
+                print("Error loading file %s "%(source_file))
+                raise
+        elif self.SOURCE == 3:  # file contains energy_eV and flux (ph/s/0.1%bw
+            source_file = self.SOURCE_FILE
+            try:
+                tmp = numpy.loadtxt(source_file)
+                energies = tmp[:,0]
+                source = tmp[:,1] * (codata.e * 1e3)
+                self.input_spectrum = source
+                script_previous = "import numpy\nimport scipy.constants as codata\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1] / (codata.e * 1e3)\n" % \
+                                (source_file)
+            except:
+                print("Error loading file %s "%(source_file))
+                raise
+
+        # substance = [self.EL1_FOR,self.EL2_FOR,self.EL3_FOR,self.EL4_FOR,self.EL5_FOR]
+        # thick     = numpy.array( (self.EL1_THI,self.EL2_THI,self.EL3_THI,self.EL4_THI,self.EL5_THI))
+        # angle     = numpy.array( (self.EL1_ANG,self.EL2_ANG,self.EL3_ANG,self.EL4_ANG,self.EL5_ANG))
+        # dens      = [self.EL1_DEN,self.EL2_DEN,self.EL3_DEN,self.EL4_DEN,self.EL5_DEN]
+        # roughness = numpy.array( (self.EL1_ROU,self.EL2_ROU,self.EL3_ROU,self.EL4_ROU,self.EL5_ROU))
+        # flags     = numpy.array( (self.EL1_FLAG,self.EL2_FLAG,self.EL3_FLAG,self.EL4_FLAG,self.EL5_FLAG))
+
+        substance = [self.EL1_FOR,  self.EL2_FOR,  self.EL3_FOR,  self.EL4_FOR,  self.EL5_FOR]  # str
+        thick     = [self.EL1_THI,  self.EL2_THI,  self.EL3_THI,  self.EL4_THI,  self.EL5_THI]  # float
+        angle     = [self.EL1_ANG,  self.EL2_ANG,  self.EL3_ANG,  self.EL4_ANG,  self.EL5_ANG]  # float
+        dens      = [self.EL1_DEN,  self.EL2_DEN,  self.EL3_DEN,  self.EL4_DEN,  self.EL5_DEN]  # str
+        roughness = [self.EL1_ROU,  self.EL2_ROU,  self.EL3_ROU,  self.EL4_ROU,  self.EL5_ROU]  # float
+        flags     = [self.EL1_FLAG, self.EL2_FLAG, self.EL3_FLAG, self.EL4_FLAG, self.EL5_FLAG] # int
+
+        # this is for creating script
+        substance_str = "["
+        thick_str = "["
+        angle_str = "["
+        dens_str = "["
+        roughness_str = "["
+        flags_str = "["
+        for i in range(self.NELEMENTS+1):
+            substance_str += "'%s'," % (substance[i])
+            thick_str     += "%g," % (thick[i])
+            angle_str     += "%g," % (angle[i])
+            dens_str      += "'%s'," % (dens[i])
+            roughness_str += "%g," % (roughness[i])
+            flags_str     += "%d," % (flags[i])
+        substance_str += "]"
+        thick_str += "]"
+        angle_str += "]"
+        dens_str += "]"
+        roughness_str += "]"
+        flags_str += "]"
+
+
+
+        if self.MATERIAL_CONSTANT_LIBRARY_FLAG == 0:
+            material_constants_library = xraylib
+            material_constants_library_str = "xraylib"
+        else:
+            material_constants_library = DabaxXraylib()
+            material_constants_library_str = 'DabaxXraylib()'
+            print(material_constants_library.info())
+
+        out_dictionary = xoppy_calc_power(
+            energies,
+            source,
+            substance                  = substance,
+            thick                      = thick    ,
+            angle                      = angle    ,
+            dens                       = dens     ,
+            roughness                  = roughness,
+            flags                      = flags    ,
+            NELEMENTS                  = self.NELEMENTS + 1,
+            FILE_DUMP                  = self.FILE_DUMP,
+            material_constants_library = material_constants_library,
+                                                )
+
+        print(out_dictionary["info"])
+
+        dict_parameters = {
+            "substance"                 : substance_str,
+            "thick"                     : thick_str,
+            "angle"                     : angle_str,
+            "dens"                      : dens_str,
+            "roughness"                 : roughness_str,
+            "flags"                     : flags_str,
+            "NELEMENTS"                 : self.NELEMENTS + 1,
+            "FILE_DUMP"                 : self.FILE_DUMP,
+            "material_constants_library": material_constants_library_str,
+        }
+
+        script_element = self.script_template().format_map(dict_parameters)
+
+        script = script_previous + script_element
+
+        self.xoppy_script.set_code(script)
+
+        return  out_dictionary, script
+
+    def script_template(self):
+        return """
+
+#
+# script to make the calculations (created by XOPPY:xpower)
+#
+
+import numpy
+from xoppylib.power.xoppy_calc_power import xoppy_calc_power
+import xraylib
+from dabax.dabax_xraylib import DabaxXraylib
+
+out_dictionary = xoppy_calc_power(
+        energy,
+        spectral_power,
+        substance = {substance},
+        thick     = {thick}, # in mm (for filters)
+        angle     = {angle}, # in mrad (for mirrors)
+        dens      = {dens},
+        roughness = {roughness}, # in A (for mirrors)
+        flags     = {flags}, # 0=Filter, 1=Mirror
+        NELEMENTS = {NELEMENTS},
+        FILE_DUMP = {FILE_DUMP},
+        material_constants_library = {material_constants_library},
+        )
+
+
+# data to pass
+energy = out_dictionary["data"][0,:]
+spectral_power = out_dictionary["data"][-1,:]
+
+#                       
+# example plots
+#
+from srxraylib.plot.gol import plot
+plot(out_dictionary["data"][0,:], out_dictionary["data"][1,:],
+    out_dictionary["data"][0,:], out_dictionary["data"][-1,:],
+    xtitle=out_dictionary["labels"][0],
+    legend=[out_dictionary["labels"][1],out_dictionary["labels"][-1]],
+    title='Spectral Power [W/eV]')
+ 
+#
+# end script
+#
+"""
+
+
 
     def extract_data_from_xoppy_output(self, calculation_output):
-        return calculation_output
+
+        out_dictionary, script = calculation_output
+
+        # send exchange
+        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
+
+        try:
+            calculated_data.add_content("xoppy_data", out_dictionary["data"].T)
+            calculated_data.add_content("plot_x_col", 0)
+            calculated_data.add_content("plot_y_col", -1)
+        except:
+            pass
+        try:
+            # print(out_dictionary["labels"])
+            calculated_data.add_content("labels", out_dictionary["labels"])
+        except:
+            pass
+        try:
+            calculated_data.add_content("info", out_dictionary["info"])
+        except:
+            pass
+
+        return calculated_data
 
 
     def get_data_exchange_widget_name(self):
@@ -661,6 +851,7 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Mu")
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Transmitivity")
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Absorption")
+                if self.do_plot_intensity(): titles.append("Spectral power absorbed in oe " + str(oe_n))
                 if self.do_plot_intensity(): titles.append("Spectral power after oe " + str(oe_n))
 
 
@@ -670,6 +861,7 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] delta/beta")
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Reflectivity-s")
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Absorption")
+                if self.do_plot_intensity(): titles.append("Spectral power absorbed in oe " + str(oe_n))
                 if self.do_plot_intensity(): titles.append("Spectral power after oe " + str(oe_n))
 
 
@@ -690,6 +882,7 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
                 if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
                 if self.do_plot_intensity(): xtitles.append("Photon Energy [eV]")
+                if self.do_plot_intensity(): xtitles.append("Photon Energy [eV]")
             else: # MIRROR
                 if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
                 if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
@@ -697,16 +890,20 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
                 if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
                 if self.do_plot_intensity(): xtitles.append("Photon Energy [eV]")
+                if self.do_plot_intensity(): xtitles.append("Photon Energy [eV]")
 
         return xtitles
 
     def getYTitles(self):
         ytitles = []
 
-        if self.SOURCE == 0:
-            if self.do_plot_intensity(): ytitles.append("Spectral Power [W/eV]")
+
+        if self.SOURCE == 1:
+            unit_str = '[a.u]'
         else:
-            if self.do_plot_intensity(): ytitles.append("Spectral Power [a.u.]")
+            unit_str = '[W/eV]'
+
+        if self.do_plot_intensity(): ytitles.append("Spectral Power %s" % unit_str )
 
 
         for oe_n in range(1, self.NELEMENTS+2):
@@ -717,20 +914,19 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] Mu cm^-1")
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] Transmitivity")
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] Absorption")
-                if self.SOURCE == 0:
-                    if self.do_plot_intensity(): ytitles.append("Spectral power [W/eV]")
-                else:
-                    if self.do_plot_intensity(): ytitles.append("Spectral power [a.u.]")
+
+                if self.do_plot_intensity(): ytitles.append("Absorbed Spectral power %s" % unit_str)
+                if self.do_plot_intensity(): ytitles.append(         "Spectral power %s" % unit_str)
+
             else: # MIRROR
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] 1-Re[n]=delta")
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] Im[n]=beta")
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] delta/beta")
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] Reflectivity-s")
                 if self.do_plot_local(): ytitles.append("[oe " + str(oe_n) + "] Transmitivity")
-                if self.SOURCE == 0:
-                    if self.do_plot_intensity(): ytitles.append("Spectral power [W/eV]")
-                else:
-                    if self.do_plot_intensity(): ytitles.append("Spectral power [a.u.]")
+
+                if self.do_plot_intensity(): ytitles.append("Absorbed Spectral power %s" % unit_str)
+                if self.do_plot_intensity(): ytitles.append(         "Spectral power %s" % unit_str)
 
         return ytitles
 
@@ -749,9 +945,9 @@ class OWxpower(XoppyWidget):
                 kind_previous = self.getKind(oe_n-1)
 
                 if kind_previous == 0: # FILTER
-                    shift += 5
-                else:
                     shift += 6
+                else:
+                    shift += 7
 
             if kind == 0: # FILTER
                 if self.do_plot_local(): variables.append((0, 2+shift))
@@ -759,6 +955,7 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): variables.append((0, 4+shift))
                 if self.do_plot_local(): variables.append((0, 5+shift))
                 if self.do_plot_intensity(): variables.append((0, 6+shift))
+                if self.do_plot_intensity(): variables.append((0, 7+shift))
             else:
                 if self.do_plot_local(): variables.append((0, 2+shift))
                 if self.do_plot_local(): variables.append((0, 3+shift))
@@ -766,6 +963,7 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): variables.append((0, 5+shift))
                 if self.do_plot_local(): variables.append((0, 6+shift))
                 if self.do_plot_intensity(): variables.append((0, 7+shift))
+                if self.do_plot_intensity(): variables.append((0, 8+shift))
 
         return variables
 
@@ -785,6 +983,7 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): logplot.append((False, False))
                 if self.do_plot_local(): logplot.append((False, False))
                 if self.do_plot_intensity(): logplot.append((False, False))
+                if self.do_plot_intensity(): logplot.append((False, False))
             else: # MIRROR
                 if self.do_plot_local(): logplot.append((False, True))
                 if self.do_plot_local(): logplot.append((False, True))
@@ -792,135 +991,59 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): logplot.append((False, False))
                 if self.do_plot_local(): logplot.append((False, False))
                 if self.do_plot_intensity(): logplot.append((False, False))
+                if self.do_plot_intensity(): logplot.append((False, False))
 
         return logplot
 
-    def xoppy_calc_xpower(self):
-
-        #
-        # prepare input for xpower_calc
-        # Note that the input for xpower_calc accepts any number of elements.
-        #
-
-        substance = [self.EL1_FOR,self.EL2_FOR,self.EL3_FOR,self.EL4_FOR,self.EL5_FOR]
-        thick     = numpy.array( (self.EL1_THI,self.EL2_THI,self.EL3_THI,self.EL4_THI,self.EL5_THI))
-        angle     = numpy.array( (self.EL1_ANG,self.EL2_ANG,self.EL3_ANG,self.EL4_ANG,self.EL5_ANG))
-        dens      = [self.EL1_DEN,self.EL2_DEN,self.EL3_DEN,self.EL4_DEN,self.EL5_DEN]
-        roughness = numpy.array( (self.EL1_ROU,self.EL2_ROU,self.EL3_ROU,self.EL4_ROU,self.EL5_ROU))
-        flags     = numpy.array( (self.EL1_FLAG,self.EL2_FLAG,self.EL3_FLAG,self.EL4_FLAG,self.EL5_FLAG))
-
-        substance = substance[0:self.NELEMENTS+1]
-        thick = thick[0:self.NELEMENTS+1]
-        angle = angle[0:self.NELEMENTS+1]
-        dens = dens[0:self.NELEMENTS+1]
-        roughness = roughness[0:self.NELEMENTS+1]
-        flags = flags[0:self.NELEMENTS+1]
-
-
-        if self.SOURCE == 0:
-            # energies = numpy.arange(0,500)
-            # elefactor = numpy.log10(10000.0 / 30.0) / 300.0
-            # energies = 10.0 * 10**(energies * elefactor)
-            # source = numpy.ones(energies.size)
-            # tmp = numpy.vstack( (energies,source))
-            if self.input_spectrum is None:
-                raise Exception("No input beam")
-            else:
-                energies = self.input_spectrum[0,:].copy()
-                source = self.input_spectrum[1,:].copy()
-        elif self.SOURCE == 1:
-            energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
-            source = numpy.ones(energies.size)
-            tmp = numpy.vstack( (energies,source))
-            self.input_spectrum = source
-        elif self.SOURCE == 2:
-            if self.SOURCE == 2: source_file = self.SOURCE_FILE
-            # if self.SOURCE == 3: source_file = "SRCOMPE"
-            # if self.SOURCE == 4: source_file = "SRCOMPF"
-            try:
-                tmp = numpy.loadtxt(source_file)
-                energies = tmp[:,0]
-                source = tmp[:,1]
-                self.input_spectrum = source
-            except:
-                print("Error loading file %s "%(source_file))
-                raise
-
-        if self.FILE_DUMP == 0:
-            output_file = None
-        else:
-            output_file = "power.spec"
-        out_dictionary = xpower_calc(energies=energies,source=source,substance=substance,
-                                     flags=flags,dens=dens,thick=thick,angle=angle,roughness=roughness,output_file=output_file)
-
-
-        try:
-            print(out_dictionary["info"])
-        except:
-            pass
-        #send exchange
-        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
-
-        try:
-            calculated_data.add_content("xoppy_data", out_dictionary["data"].T)
-            calculated_data.add_content("plot_x_col",0)
-            calculated_data.add_content("plot_y_col",-1)
-        except:
-            pass
-        try:
-            # print(out_dictionary["labels"])
-            calculated_data.add_content("labels", out_dictionary["labels"])
-        except:
-            pass
-        try:
-            calculated_data.add_content("info",out_dictionary["info"])
-        except:
-            pass
-
-        return calculated_data
-
-
 if __name__ == "__main__":
+    import sys
+    input_type = 0
 
+    if input_type == 1:
+        from oasys.widgets.exchange import DataExchangeObject
+        input_data_type = "POWER"
 
-    from oasys.widgets.exchange import DataExchangeObject
+        if input_data_type == "POWER":
+            # create fake UNDULATOR_FLUX xoppy exchange data
+            e = numpy.linspace(1000.0, 10000.0, 100)
+            source = e/10
+            received_data = DataExchangeObject("XOPPY", "POWER")
+            received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
+            received_data.add_content("xoppy_code", "US")
 
+        elif input_data_type == "POWER3D":
+            # create unulator_radiation xoppy exchange data
+            from xoppylib.xoppy_undulators import xoppy_calc_undulator_radiation
 
-
-    input_data_type = "POWER"
-
-    if input_data_type == "POWER":
-        # create fake UNDULATOR_FLUX xoppy exchange data
-        e = numpy.linspace(1000.0, 10000.0, 100)
-        source = e/10
-        received_data = DataExchangeObject("XOPPY", "POWER")
-        received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
-        received_data.add_content("xoppy_code", "US")
-
-    elif input_data_type == "POWER3D":
-        # create unulator_radiation xoppy exchange data
-        from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_radiation
-
-        e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
-                                           ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
-                                           ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
-                                           PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
-                                           SETRESONANCE=0,HARMONICNUMBER=1,
-                                           GAPH=0.001,GAPV=0.001,\
-                                           HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
-                                           PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
-                                           USEEMITTANCES=1)
-        received_data = DataExchangeObject("XOPPY", "POWER3D")
-        received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
-        received_data.add_content("xoppy_data", [p, e, h, v])
-        received_data.add_content("xoppy_code", code)
+            e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
+                                               ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
+                                               ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
+                                               PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
+                                               SETRESONANCE=0,HARMONICNUMBER=1,
+                                               GAPH=0.001,GAPV=0.001,\
+                                               HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
+                                               PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
+                                               USEEMITTANCES=1)
+            received_data = DataExchangeObject("XOPPY", "POWER3D")
+            received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
+            received_data.add_content("xoppy_data", [p, e, h, v])
+            received_data.add_content("xoppy_code", code)
 
 
 
 
-    app = QApplication(sys.argv)
-    w = OWxpower()
-    w.acceptExchangeData(received_data)
-    w.show()
-    app.exec()
-    w.saveSettings()
\ No newline at end of file
+        app = QApplication(sys.argv)
+        w = OWxpower()
+        w.acceptExchangeData(received_data)
+        w.show()
+        app.exec()
+        w.saveSettings()
+
+    else:
+        app = QApplication(sys.argv)
+        w = OWxpower()
+        w.SOURCE = 1
+        w.show()
+        app.exec()
+        w.saveSettings()
+
