@@ -10,7 +10,7 @@ from oasys.widgets.exchange import DataExchangeObject
 
 
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
-from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_spectrum
+from xoppylib.sources.xoppy_undulators import xoppy_calc_undulator_spectrum
 
 from syned.widget.widget_decorator import WidgetDecorator
 import syned.beamline.beamline as synedb
@@ -354,18 +354,18 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
         }
 
 
-        # print(self.script_template().format_map(dict_parameters))
+        script = self.script_template().format_map(dict_parameters)
 
-        self.xoppy_script.set_code(self.script_template().format_map(dict_parameters))
+        self.xoppy_script.set_code(script)
 
-        return energy, flux, spectral_power, cumulated_power
+        return energy, flux, spectral_power, cumulated_power, script
 
     def script_template(self):
         return """
 #
 # script to make the calculations (created by XOPPY:undulator_spectrum)
 #
-from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_spectrum
+from xoppylib.sources.xoppy_undulators import xoppy_calc_undulator_spectrum
 energy, flux, spectral_power, cumulated_power = xoppy_calc_undulator_spectrum(
     ELECTRONENERGY={ELECTRONENERGY},
     ELECTRONENERGYSPREAD={ELECTRONENERGYSPREAD},
@@ -389,13 +389,20 @@ energy, flux, spectral_power, cumulated_power = xoppy_calc_undulator_spectrum(
     PHOTONENERGYPOINTS={PHOTONENERGYPOINTS},
     METHOD={METHOD},
     USEEMITTANCES={USEEMITTANCES})
+
+#
 # example plot
+#
 from srxraylib.plot.gol import plot
-plot(energy,flux,ytitle="Flux [photons/s/o.1%bw]",xtitle="Poton energy [eV]",title="Undulator Flux",
+
+plot(energy,flux,
+    xtitle="Photon energy [eV]",ytitle="Flux [photons/s/o.1%bw]",title="Undulator Flux",
     xlog=False,ylog=False,show=False)
-plot(energy,spectral_power,ytitle="Power [W/eV]",xtitle="Poton energy [eV]",title="Undulator Spectral Power",
+plot(energy,spectral_power,
+    xtitle="Photon energy [eV]",ytitle="Power [W/eV]",title="Undulator Spectral Power",
     xlog=False,ylog=False,show=False)
-plot(energy,cumulated_power,ytitle="Cumulated Power [W]",xtitle="Poton energy [eV]",title="Undulator Cumulated Power",
+plot(energy,cumulated_power,
+    xtitle="Photon energy [eV]",ytitle="Cumulated Power [W]",title="Undulator Cumulated Power",
     xlog=False,ylog=False,show=True)
 #
 # end script
@@ -403,7 +410,7 @@ plot(energy,cumulated_power,ytitle="Cumulated Power [W]",xtitle="Poton energy [e
 """
 
     def extract_data_from_xoppy_output(self, calculation_output):
-        e, f, sp, csp = calculation_output
+        e, f, sp, csp, script = calculation_output
 
         data = numpy.zeros((len(e), 4))
         data[:, 0] = numpy.array(e)
@@ -413,6 +420,8 @@ plot(energy,cumulated_power,ytitle="Cumulated Power [W]",xtitle="Poton energy [e
 
         calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
         calculated_data.add_content("xoppy_data", data)
+        calculated_data.add_content("xoppy_script", script)
+
 
         return calculated_data
 
